@@ -9,41 +9,45 @@ import auto_editor.utils.func as usefulfunctions
 from auto_editor.edit import edit_media
 from auto_editor.ffwrapper import FFmpeg
 from auto_editor.utils.log import Log, Timer
-from auto_editor.utils.types import MainArgs
+from auto_editor.utils.types import (
+    MainArgs,
+    color,
+    margin,
+    number,
+    resolution,
+    sample_rate,
+    speed_range,
+    time,
+    time_range,
+)
 from auto_editor.validate_input import valid_input
 from auto_editor.vanparse import ArgumentParser
 
 
 def main_options(parser: ArgumentParser) -> ArgumentParser:
-    from auto_editor.utils.types import (
-        color,
-        margin,
-        number,
-        sample_rate,
-        speed_range,
-        time,
-        time_range,
-    )
-
     parser.add_text("Object Options")
     parser.add_argument(
         "--add-text",
         nargs="*",
+        pool=True,
         help="Add a text object to the timeline.",
     )
     parser.add_argument(
         "--add-rectangle",
         nargs="*",
+        pool=True,
         help="Add a rectangle object to the timeline.",
     )
     parser.add_argument(
         "--add-ellipse",
         nargs="*",
+        pool=True,
         help="Add an ellipse object to the timeline.",
     )
     parser.add_argument(
         "--add-image",
         nargs="*",
+        pool=True,
         help="Add an image object onto the timeline.",
     )
     parser.add_text("URL Download Options")
@@ -62,13 +66,13 @@ def main_options(parser: ArgumentParser) -> ArgumentParser:
         "--video-codec",
         "-vcodec",
         "-c:v",
-        help="Set the video codec for the output media file.",
+        help="Set video codec for output media.",
     )
     parser.add_argument(
         "--audio-codec",
         "-acodec",
         "-c:a",
-        help="Set the audio codec for the output media file.",
+        help="Set audio codec for output media.",
     )
     parser.add_argument(
         "--video-bitrate",
@@ -139,7 +143,7 @@ def main_options(parser: ArgumentParser) -> ArgumentParser:
         "--frame-rate",
         "-fps",
         "-r",
-        type=float,
+        type=number,
         help="Set the frame rate for the timeline and output media.",
     )
     parser.add_argument(
@@ -148,8 +152,12 @@ def main_options(parser: ArgumentParser) -> ArgumentParser:
         type=sample_rate,
         help="Set the sample rate for the timeline and output media.",
     )
+    parser.add_argument(
+        "--resolution", "-res", type=resolution, help="Set timeline width and height."
+    )
     parser.add_argument(
         "--background",
+        "-b",
         type=color,
         help="Set the color of the background that is visible when the video is moved.",
     )
@@ -179,8 +187,11 @@ def main_options(parser: ArgumentParser) -> ArgumentParser:
         help="Choose the export mode.",
     )
     parser.add_text("Utility Options")
+    parser.add_argument("--player", "-p", help="Set player to open output media files.")
     parser.add_argument(
-        "--no-open", flag=True, help="Do not open the file after editing is done."
+        "--no-open",
+        flag=True,
+        help="Do not open the output file after editing is done.",
     )
     parser.add_argument(
         "--temp-dir",
@@ -297,6 +308,7 @@ def main() -> None:
             ],
         )
 
+    log = Log(args.debug, args.quiet)
     timer = Timer(args.quiet)
 
     exporting_to_editor = args.export in ("premiere", "final-cut-pro", "shotcut")
@@ -321,17 +333,17 @@ def main() -> None:
         args.quiet = True
 
     if args.input == []:
-        Log().error("You need to give auto-editor an input file.")
+        log.error("You need to give auto-editor an input file.")
 
     if args.temp_dir is None:
         temp = tempfile.mkdtemp()
     else:
         temp = args.temp_dir
         if os.path.isfile(temp):
-            Log().error("Temp directory cannot be an already existing file.")
+            log.error("Temp directory cannot be an already existing file.")
         if os.path.isdir(temp):
             if len(os.listdir(temp)) != 0:
-                Log().error("Temp directory should be empty!")
+                log.error("Temp directory should be empty!")
         else:
             os.mkdir(temp)
 
@@ -377,7 +389,13 @@ def main() -> None:
             timer.stop()
 
         if not args.no_open and output is not None:
-            usefulfunctions.open_with_system_default(output, log)
+            if args.player is None:
+                usefulfunctions.open_with_system_default(output, log)
+            else:
+                import subprocess
+                from shlex import split
+
+                subprocess.run(split(args.player) + [output])
     except KeyboardInterrupt:
         log.error("Keyboard Interrupt")
     log.cleanup()
