@@ -1,12 +1,12 @@
 import os
-from typing import List, Optional, Type
+from typing import List, Optional
 
 from auto_editor.ffwrapper import FFmpeg, FileInfo
 from auto_editor.timeline import Timeline, make_timeline
-from auto_editor.utils.container import get_rules
+from auto_editor.utils.container import container_constructor
 from auto_editor.utils.log import Log
 from auto_editor.utils.progressbar import ProgressBar
-from auto_editor.utils.types import Chunk, Chunks, MainArgs
+from auto_editor.utils.types import Args, Chunk, Chunks
 
 
 def set_output_name(path: str, inp_ext: str, export: str) -> str:
@@ -29,59 +29,63 @@ def set_output_name(path: str, inp_ext: str, export: str) -> str:
 
 
 def edit_media(
-    path_list: List[str], ffmpeg: FFmpeg, args: Type[MainArgs], temp: str, log: Log
+    paths: List[str], ffmpeg: FFmpeg, args: Args, temp: str, log: Log
 ) -> Optional[str]:
 
     progress = ProgressBar(args.progress)
     timeline = None
 
-    path_ext = os.path.splitext(path_list[0])[1]
-    if path_ext == ".json":
-        from auto_editor.formats.json import read_json
+    if paths:
+        path_ext = os.path.splitext(paths[0])[1]
+        if path_ext == ".json":
+            from auto_editor.formats.json import read_json
 
-        timeline = read_json(path_list[0], ffmpeg, log)
-        inputs: List[FileInfo] = timeline.inputs
+            timeline = read_json(paths[0], ffmpeg, log)
+            inputs: List[FileInfo] = timeline.inputs
+        else:
+            inputs = [FileInfo(path, ffmpeg, log) for path in paths]
     else:
         inputs = []
-        for path in path_list:
-            inputs.append(FileInfo(path, ffmpeg, log))
+    del paths
 
-    del path_list
-    inp = inputs[0]
-
-    if args.output_file is None:
-        output = set_output_name(inp.path, inp.ext, args.export)
+    if inputs:
+        inp = inputs[0]
+        if args.output_file is None:
+            output = set_output_name(inp.path, inp.ext, args.export)
+        else:
+            output = args.output_file
+            if os.path.splitext(output)[1] == "":
+                output = set_output_name(output, inp.ext, args.export)
+        del inp
     else:
-        output = args.output_file
-        if os.path.splitext(output)[1] == "":
-            output = set_output_name(output, inp.ext, args.export)
+        output = "out.mp4" if args.output_file is None else args.output_file
 
-    output_container = os.path.splitext(output)[1].replace(".", "")
+    out_ext = os.path.splitext(output)[1].replace(".", "")
 
     # Check if export options make sense.
-    rules = get_rules(output_container)
+    ctr = container_constructor(out_ext)
     codec_error = "'{}' codec is not supported in '{}' container."
 
-    if not (args.sample_rate is None or args.sample_rate == "unset"):
-        if rules.samplerate is not None and args.sample_rate not in rules.samplerate:
-            log.error(
-                f"'{output_container}' container only supports samplerates: {rules.samplerate}"
-            )
+    if ctr.samplerate is not None and args.sample_rate not in ctr.samplerate:
+        log.error(f"'{out_ext}' container only supports samplerates: {ctr.samplerate}")
 
     vcodec = args.video_codec
     if vcodec == "uncompressed":
         vcodec = "mpeg4"
     if vcodec == "copy":
-        vcodec = inp.videos[0].codec
+        if inputs:
+            vcodec = inputs[0].videos[0].codec
+        else:
+            log.error("No input video to copy its codec from.")
 
     if vcodec != "auto":
-        if rules.vstrict:
-            assert rules.vcodecs is not None
-            if vcodec not in rules.vcodecs:
-                log.error(codec_error.format(vcodec, output_container))
+        if ctr.vstrict:
+            assert ctr.vcodecs is not None
+            if vcodec not in ctr.vcodecs:
+                log.error(codec_error.format(vcodec, out_ext))
 
-        if vcodec in rules.disallow_v:
-            log.error(codec_error.format(vcodec, output_container))
+        if vcodec in ctr.disallow_v:
+            log.error(codec_error.format(vcodec, out_ext))
 
     acodec = args.audio_codec
     if acodec == "copy":
@@ -89,18 +93,16 @@ def edit_media(
         log.debug(f"Settings acodec to {acodec}")
 
     if acodec not in ("unset", "auto"):
-        if rules.astrict:
-            assert rules.acodecs is not None
-            if acodec not in rules.acodecs:
-                log.error(codec_error.format(acodec, output_container))
+        if ctr.astrict:
+            assert ctr.acodecs is not None
+            if acodec not in ctr.acodecs:
+                log.error(codec_error.format(acodec, out_ext))
 
-        if acodec in rules.disallow_a:
-            log.error(codec_error.format(acodec, output_container))
+        if acodec in ctr.disallow_a:
+            log.error(codec_error.format(acodec, out_ext))
 
-    if args.keep_tracks_separate and rules.max_audios == 1:
-        log.warning(
-            f"'{output_container}' container doesn't support multiple audio tracks."
-        )
+    if args.keep_tracks_separate and ctr.max_audios == 1:
+        log.warning(f"'{out_ext}' container doesn't support multiple audio tracks.")
 
     if not args.preview and not args.timeline:
         if os.path.isdir(output):
@@ -111,13 +113,15 @@ def edit_media(
             os.remove(output)
 
     # Extract subtitles in their native format.
-    if len(inp.subtitles) > 0:
+    if inputs and len(inputs[0].subtitles) > 0:
+        inp = inputs[0]
         cmd = ["-i", inp.path, "-hide_banner"]
         for s, sub in enumerate(inp.subtitles):
             cmd.extend(["-map", f"0:s:{s}"])
         for s, sub in enumerate(inp.subtitles):
             cmd.extend([os.path.join(temp, f"{s}s.{sub.ext}")])
         ffmpeg.run(cmd)
+        del inp
 
     log.conwrite("Extracting audio")
 
@@ -127,7 +131,10 @@ def edit_media(
     cmd.append("-hide_banner")
 
     if args.sample_rate is None:
-        samplerate = inputs[0].get_samplerate()
+        if inputs:
+            samplerate = inputs[0].get_samplerate()
+        else:
+            samplerate = 48000
     else:
         samplerate = args.sample_rate
 
@@ -188,42 +195,52 @@ def edit_media(
         shotcut_xml(output, timeline)
         return output
 
-    def make_media(inp: FileInfo, timeline: Timeline, output: str) -> None:
+    def make_media(timeline: Timeline, output: str) -> None:
         from auto_editor.output import mux_quality_media
         from auto_editor.render.video import render_av
 
-        if rules.allow_subtitle:
+        visual_output = []
+        audio_output = []
+        apply_later = False
+        inp = timeline.inputs[0]
+
+        if ctr.allow_subtitle:
             from auto_editor.render.subtitle import cut_subtitles
 
             cut_subtitles(ffmpeg, timeline, temp, log)
 
-        if rules.allow_audio:
+        if ctr.allow_audio:
             from auto_editor.render.audio import make_new_audio
 
-            for t in range(len(inp.audios)):
-                make_new_audio(t, temp, timeline, progress, log)
+            audio_output = make_new_audio(timeline, progress, temp, log)
 
-        video_output = []
-
-        if rules.allow_video:
+        if ctr.allow_video:
             for v, vid in enumerate(inp.videos):
                 if vid.codec not in ("png", "mjpeg", "webp") and v == 0:
                     out_path, apply_later = render_av(
-                        ffmpeg, timeline, args, progress, rules, temp, log
+                        ffmpeg, timeline, args, progress, ctr, temp, log
                     )
-                    video_output.append((v, True, out_path, apply_later))
-                elif rules.allow_image:
+                    visual_output.append((True, out_path))
+                elif ctr.allow_image:
                     out_path = os.path.join(temp, f"{v}.{vid.codec}")
                     # fmt: off
                     ffmpeg.run(["-i", inp.path, "-map", "0:v", "-map", "-0:V",
                         "-c", "copy", out_path])
                     # fmt: on
-                    video_output.append((v, False, out_path, False))
+                    visual_output.append((False, out_path))
 
         log.conwrite("Writing output file")
-
         mux_quality_media(
-            ffmpeg, video_output, rules, output, output_container, args, inp, temp, log
+            ffmpeg,
+            visual_output,
+            audio_output,
+            apply_later,
+            ctr,
+            output,
+            args,
+            inp,
+            temp,
+            log,
         )
 
     if args.export == "clip-sequence":
@@ -235,11 +252,9 @@ def edit_media(
         from auto_editor.timeline import clipify, make_av
         from auto_editor.utils.func import append_filename
 
-        def pad_chunk(chunk: Chunk, total_frames: int) -> Chunks:
+        def pad_chunk(chunk: Chunk, total: int) -> Chunks:
             start = [] if chunk[0] == 0 else [(0, chunk[0], 99999.0)]
-            end = (
-                [] if chunk[1] == total_frames else [(chunk[1], total_frames, 99999.0)]
-            )
+            end = [] if chunk[1] == total else [(chunk[1], total, 99999.0)]
             return start + [chunk] + end
 
         clip_num = 0
@@ -248,7 +263,7 @@ def edit_media(
                 continue
 
             _c = pad_chunk(chunk, total_frames)
-            vspace, aspace = make_av(clipify(_c, 0), inp)
+            vspace, aspace = make_av([clipify(_c, 0, 0)], [inp])
             my_timeline = Timeline(
                 timeline.inputs,
                 timeline.fps,
@@ -260,12 +275,8 @@ def edit_media(
                 _c,
             )
 
-            make_media(
-                inp,
-                my_timeline,
-                append_filename(output, f"-{clip_num}"),
-            )
+            make_media(my_timeline, append_filename(output, f"-{clip_num}"))
             clip_num += 1
     else:
-        make_media(inp, timeline, output)
+        make_media(timeline, output)
     return output
