@@ -1,21 +1,29 @@
 import os.path
 import subprocess
 from dataclasses import dataclass
+from math import ceil
 from typing import Dict, List, Tuple, Union
 
+import av
+from PIL import Image, ImageChops, ImageDraw, ImageFont, ImageOps
+
 from auto_editor.ffwrapper import FFmpeg
 from auto_editor.objects import EllipseObj, ImageObj, RectangleObj, TextObj, VideoObj
 from auto_editor.output import get_vcodec, video_quality
 from auto_editor.timeline import Timeline, Visual
+from auto_editor.utils.container import Container
 from auto_editor.utils.encoder import encoders
 from auto_editor.utils.log import Log
 from auto_editor.utils.progressbar import ProgressBar
+from auto_editor.utils.types import Args
+
+av.logging.set_level(av.logging.PANIC)
 
 
 @dataclass
 class VideoFrame:
     index: int
-    source: int
+    src: int
 
 
 # From: github.com/PyAV-Org/PyAV/blob/main/av/video/frame.pyx
@@ -92,53 +100,63 @@ def one_pos_two_pos(
 def render_av(
     ffmpeg: FFmpeg,
     timeline: Timeline,
-    args,
+    args: Args,
     progress: ProgressBar,
-    rules,
+    ctr: Container,
     temp: str,
     log: Log,
 ) -> Tuple[str, bool]:
-    try:
-        import av
 
-        av.logging.set_level(av.logging.PANIC)
-    except ImportError:
-        log.import_error("av")
-    try:
-        from PIL import Image, ImageChops, ImageDraw, ImageFont
-    except ImportError:
-        log.import_error("Pillow")
+    FontCache = Dict[
+        str, Tuple[Union[ImageFont.FreeTypeFont, ImageFont.ImageFont], float]
+    ]
 
-    font_cache: Dict[str, Union[ImageFont.FreeTypeFont, ImageFont.ImageFont]] = {}
-    img_cache = {}
+    font_cache: FontCache = {}
+    img_cache: Dict[str, Image.Image] = {}
     for layer in timeline.v:
-        for obj in layer:
-            if isinstance(obj, TextObj) and obj.font not in font_cache:
+        for vobj in layer:
+            if isinstance(vobj, TextObj) and (vobj.font, vobj.size) not in font_cache:
                 try:
-                    if obj.font == "default":
-                        font_cache[obj.font] = ImageFont.load_default()
+                    if vobj.font == "default":
+                        font_cache[(vobj.font, vobj.size)] = ImageFont.load_default()
                     else:
-                        font_cache[obj.font] = ImageFont.truetype(obj.font, obj.size)
+                        font_cache[(vobj.font, vobj.size)] = ImageFont.truetype(
+                            vobj.font, vobj.size
+                        )
                 except OSError:
-                    log.error(f"Font '{obj.font}' not found.")
+                    log.error(f"Font '{vobj.font}' not found.")
 
-            if isinstance(obj, ImageObj) and obj.src not in img_cache:
-                source = Image.open(obj.src)
+            if isinstance(vobj, ImageObj) and vobj.src not in img_cache:
+                source = Image.open(vobj.src)
                 source = source.convert("RGBA")
-                source = source.rotate(obj.rotate, expand=True)
+                source = source.rotate(vobj.rotate, expand=True)
                 source = ImageChops.multiply(
                     source,
                     Image.new(
-                        "RGBA", source.size, (255, 255, 255, int(obj.opacity * 255))
+                        "RGBA", source.size, (255, 255, 255, int(vobj.opacity * 255))
                     ),
                 )
-                img_cache[obj.src] = source
+                img_cache[vobj.src] = source
 
     inp = timeline.inp
-    cn = av.open(inp.path, "r")
-    pix_fmt = cn.streams.video[0].pix_fmt
-    target_pix_fmt = pix_fmt if pix_fmt in allowed_pix_fmt else "yuv420p"
-    my_codec = get_vcodec(args.video_codec, inp, rules)
+    cns = [av.open(inp.path, "r") for inp in timeline.inputs]
+
+    decoders = []
+    tous = []
+    pix_fmts = []
+    for cn in cns:
+        stream = cn.streams.video[0]
+        stream.thread_type = "AUTO"
+
+        tous.append(int(stream.time_base.denominator / stream.average_rate))
+        pix_fmts.append(stream.pix_fmt)
+        decoders.append(cn.decode(stream))
+
+    log.debug(f"Tous: {tous}")
+    log.debug(f"Clips: {timeline.v}")
+
+    target_pix_fmt = pix_fmts[0] if pix_fmts[0] in allowed_pix_fmt else "yuv420p"
+    my_codec = get_vcodec(args.video_codec, inp, ctr)
 
     apply_video_later = True
 
@@ -150,9 +168,7 @@ def render_av(
 
     log.debug(f"apply video quality settings now: {not apply_video_later}")
 
-    width = inp.videos[0].width
-    height = inp.videos[0].height
-
+    width, height = timeline.res
     spedup = os.path.join(temp, "spedup0.mp4")
 
     cmd = [
@@ -177,7 +193,7 @@ def render_av(
     if apply_video_later:
         cmd.extend(["-c:v", "mpeg4", "-qscale:v", "1"])
     else:
-        cmd = video_quality(cmd, args, inp, rules)
+        cmd = video_quality(cmd, args, inp, ctr)
 
     cmd.append(spedup)
 
@@ -186,11 +202,6 @@ def render_av(
     )
     assert process2.stdin is not None
 
-    stream = cn.streams.video[0]
-    stream.thread_type = "AUTO"
-    tou = int(stream.time_base.denominator / stream.average_rate)
-    log.debug(f"Tou: {tou}")
-
     seek = 10
     seek_frame = None
     frames_saved = 0
@@ -199,45 +210,43 @@ def render_av(
     if args.no_seek:
         SEEK_COST = 4294967295
     else:
-        SEEK_COST = int(stream.average_rate * 5)
+        SEEK_COST = int(cns[0].streams.video[0].average_rate * 5)
     SEEK_RETRY = SEEK_COST // 2
 
-    # New NLE method setup
-    end = 0
-    for layer in timeline.v:
-        end = max(end, layer[-1].start + layer[-1].dur)
-
-    d = cn.decode(stream)
-    progress.start(end, "Creating new video")
+    progress.start(timeline.end, "Creating new video")
 
     null_img = Image.new("RGB", (width, height), args.background)
-    null_frame = av.VideoFrame.from_image(null_img).reformat(format=pix_fmt)
+    null_frame = av.VideoFrame.from_image(null_img).reformat(format=target_pix_fmt)
 
     frame_index = -1
     try:
-        for index in range(end):
+        for index in range(timeline.end):
             # Add objects to obj_list
             obj_list: List[Union[VideoFrame, Visual]] = []
             for layer in timeline.v:
-                for obj in layer:
-                    if isinstance(obj, VideoObj):
-                        if index >= obj.start and index < obj.start + round(
-                            obj.speed * obj.dur
+                for lobj in layer:
+                    if isinstance(lobj, VideoObj):
+                        if index >= lobj.start and index < lobj.start + ceil(
+                            lobj.dur / lobj.speed
                         ):
                             obj_list.append(
-                                VideoFrame(obj.offset + index - obj.start, obj.src)
+                                VideoFrame(
+                                    lobj.offset
+                                    + round((index - lobj.start) * lobj.speed),
+                                    lobj.src,
+                                )
                             )
-                    elif index >= obj.start and index < obj.start + obj.dur:
-                        obj_list.append(obj)
+                    elif index >= lobj.start and index < lobj.start + lobj.dur:
+                        obj_list.append(lobj)
 
             # Render obj_list
             frame = null_frame
             for obj in obj_list:
                 if isinstance(obj, VideoFrame):
                     if frame_index > obj.index:
-                        log.debug("Seeking to beginning of file")
-                        cn.seek(0)
-                        frame = next(d)
+                        log.debug(f"Seek: {frame_index} -> beginning")
+                        cns[obj.src].seek(0)
+                        frame = next(decoders[obj.src])
                         frame_index = round(frame.time * timeline.fps)
 
                     while frame_index < obj.index:
@@ -245,25 +254,36 @@ def render_av(
                         if obj.index - frame_index > SEEK_COST and frame_index > seek:
                             seek = frame_index + SEEK_RETRY
                             seek_frame = frame_index
-                            log.debug(f"Seeking to {frame_index * tou}")
-                            cn.seek(frame_index * tou, stream=stream)
+                            log.debug(f"Seek: {frame_index} -> {obj.index}")
+                            cns[obj.src].seek(
+                                obj.index * tous[obj.src],
+                                stream=cns[obj.src].streams.video[0],
+                            )
 
                         try:
-                            frame = next(d)
+                            frame = next(decoders[obj.src])
                             frame_index = round(frame.time * timeline.fps)
                         except StopIteration:
-                            log.warning(
-                                "No source frame at current position. Using null frame"
-                            )
+                            log.debug(f"No source frame at {index=}. Using null frame")
                             frame = null_frame
                             break
 
-                        if frame.key_frame:
-                            log.debug(f"Keyframe {frame_index} {frame.pts}")
                         if seek_frame is not None:
-                            log.debug(f"Skipped {frame_index - seek_frame} frames")
+                            log.debug(
+                                f"Skipped {frame_index - seek_frame} frame indexes"
+                            )
                             frames_saved += frame_index - seek_frame
                             seek_frame = None
+                        if frame.key_frame:
+                            log.debug(f"Keyframe {frame_index} {frame.pts}")
+
+                    if frame.width != width or frame.height != height:
+                        img = frame.to_image().convert("RGB")
+                        if img.width > width or img.height > height:
+                            factor = ceil(max(width / img.width, height / img.height))
+                            img = ImageOps.scale(img, factor)
+                        img = ImageOps.pad(img, (width, height), color=args.background)
+                        frame = frame.from_image(img).reformat(format=target_pix_fmt)
 
                 # Render visual objects
                 else:
@@ -273,13 +293,13 @@ def render_av(
 
                     if isinstance(obj, TextObj):
                         text_w, text_h = draw.textsize(
-                            obj.content, font=font_cache[obj.font]
+                            obj.content, font=font_cache[(obj.font, obj.size)]
                         )
                         pos = apply_anchor(obj.x, obj.y, text_w, text_h, "ce")
                         draw.text(
                             pos,
                             obj.content,
-                            font=font_cache[obj.font],
+                            font=font_cache[(obj.font, obj.size)],
                             fill=obj.fill,
                             align=obj.align,
                             stroke_width=obj.stroke,
@@ -312,7 +332,7 @@ def render_av(
                         obj_img.paste(img_cache[obj.src], pos)
 
                     img = Image.alpha_composite(img, obj_img)
-                    frame = frame.from_image(img).reformat(format=pix_fmt)
+                    frame = frame.from_image(img).reformat(format=target_pix_fmt)
 
             process2.stdin.write(frame.to_ndarray().tobytes())
 
@@ -332,14 +352,9 @@ def render_av(
     if args.scale != 1:
         sped_input = os.path.join(temp, "spedup0.mp4")
         spedup = os.path.join(temp, "scale0.mp4")
+        scale_filter = f"scale=iw*{args.scale}:ih*{args.scale}"
 
-        cmd = [
-            "-i",
-            sped_input,
-            "-vf",
-            f"scale=iw*{args.scale}:ih*{args.scale}",
-            spedup,
-        ]
+        cmd = ["-i", sped_input, "-vf", scale_filter, spedup]
 
         check_errors = ffmpeg.pipe(cmd)
         if "Error" in check_errors or "failed" in check_errors:
