@@ -263,19 +263,12 @@ def main(sys_args: Optional[List[str]] = None):
 
     # Issue #184
     def unit_tests():
-        """
-        Make sure all units are working appropriately. That includes:
-
-         - Seconds units: s, sec, secs, second, seconds
-         - Frame units:   f, frame, frames
-         - Sample units:  Hz, kHz
-         - Percent:       %
-        """
-
         run_program(
             ["example.mp4", "--mark_as_loud", "20s,22sec", "25secs,26.5seconds"]
         )
-        run_program(["example.mp4", "--sample_rate", "44100"])
+        run_program(["example.mp4", "--edit", "all", "--set-speed", "125%,-30,end"])
+        run_program(["example.mp4", "--sample_rate", "44_100"])
+        run_program(["example.mp4", "--margin", "3_0"])
         run_program(["example.mp4", "--sample_rate", "44100 Hz"])
         run_program(["example.mp4", "--sample_rate", "44.1 kHz"])
         run_program(["example.mp4", "--silent_threshold", "4%"])
@@ -367,7 +360,7 @@ def main(sys_args: Optional[List[str]] = None):
         run_program(["example.mp4", "--export_as_json"])
         run_program(["example.json"])
 
-    def scale_tests():
+    def resolution_and_scale():
         run_program(["example.mp4", "--scale", "1.5"])
         with av.open("example_ALTERED.mp4") as cn:
             assert cn.streams.video[0].average_rate == 30
@@ -382,13 +375,19 @@ def main(sys_args: Optional[List[str]] = None):
             assert cn.streams.video[0].height == 144
             assert cn.streams.audio[0].rate == 48000
 
+        run_program(["example.mp4", "-res", "700,380", "-b", "darkgreen"])
+        with av.open("example_ALTERED.mp4") as cn:
+            assert cn.streams.video[0].average_rate == 30
+            assert cn.streams.video[0].width == 700
+            assert cn.streams.video[0].height == 380
+            assert cn.streams.audio[0].rate == 48000
+
     def various_errors_test():
         check_for_error(
             ["example.mp4", "--add_rectangle", "0,60", "--cut_out", "60,end"]
         )
 
-    def effect_tests():
-        """Test rendering video objects"""
+    def render_video_objs():
         run_program(
             [
                 "resources/testsrc.mp4",
@@ -400,12 +399,43 @@ def main(sys_args: Optional[List[str]] = None):
         )
         os.remove("resources/testsrc_ALTERED.mp4")
 
+        # Every element should be visible, order should be preserved.
+        run_program(
+            [
+                "example.mp4",
+                "--add-ellipse",
+                "0,30,50%,50%,300,300,fill=red",
+                "--add-rectangle",
+                "0,30,500,440,400,200,fill=skyblue",
+                "--add-ellipse",
+                "0,30,50%,50%,100,100,fill=darkgreen",
+                "--edit",
+                "none",
+                "--cut-out",
+                "30,end",
+            ]
+        )
+
+        # Both ellipses should be visible
+        run_program(
+            [
+                "example.mp4",
+                "--add-ellipse",
+                "0,60,50%,50%,300,300,fill=darkgreen",
+                "0,30,50%,50%,200,200,fill=green",
+                "--edit",
+                "none",
+                "--cut-out",
+                "60,end",
+            ]
+        )
+
     def render_text():
         run_program(["example.mp4", "--add-text", "0,30,This is my text,font=default"])
 
     def check_font_error():
         check_for_error(
-            ["example.mp4", "--add-text", "0,30,text,0,0,30,notafont"], "not found"
+            ["example.mp4", "--add-text", "0,30,text,0,0,notafont"], "not found"
         )
 
     def export_tests():
@@ -438,6 +468,52 @@ def main(sys_args: Optional[List[str]] = None):
         run_program(["example.mp4", "hmm.mp4", "--combine-files", "--debug"])
         os.remove("hmm.mp4")
 
+    # Issue #241
+    def multi_track_edit():
+        run_program(
+            [
+                "example.mp4",
+                "resources/multi-track.mov",
+                "--edit",
+                "audio:stream=1",
+                "-o",
+                "out.mov",
+            ]
+        )
+        with av.open("out.mov", "r") as cn:
+            assert len(cn.streams.audio) == 1
+
+    def concat_mux_tracks():
+        run_program(["example.mp4", "resources/multi-track.mov", "-o", "out.mov"])
+        with av.open("out.mov", "r") as cn:
+            assert len(cn.streams.audio) == 1
+
+    def concat_multiple_tracks():
+        run_program(
+            [
+                "resources/multi-track.mov",
+                "resources/multi-track.mov",
+                "--keep-tracks-separate",
+                "-o",
+                "out.mov",
+            ]
+        )
+        with av.open("out.mov", "r") as cn:
+            assert len(cn.streams.audio) == 2, f"audio streams: {len(cn.streams.audio)}"
+
+        run_program(
+            [
+                "example.mp4",
+                "resources/multi-track.mov",
+                "--keep-tracks-separate",
+                "-o",
+                "out.mov",
+            ]
+        )
+        with av.open("out.mov", "r") as cn:
+            assert len(cn.streams.audio) == 2
+        os.remove("out.mov")
+
     def frame_rate():
         run_program(["example.mp4", "-r", "15", "--no-seek"])
         with av.open("example_ALTERED.mp4", "r") as cn:
@@ -566,10 +642,16 @@ def main(sys_args: Optional[List[str]] = None):
     if args.category in ("cli", "all"):
         tests.extend(
             [
+                edit_positive_tests,
+                edit_negative_tests,
+                multi_track_edit,
+                concat_mux_tracks,
+                concat_multiple_tracks,
+                render_video_objs,
+                resolution_and_scale,
                 various_errors_test,
                 render_text,
                 check_font_error,
-                effect_tests,
                 frame_rate,
                 help_tests,
                 version_test,
@@ -590,11 +672,8 @@ def main(sys_args: Optional[List[str]] = None):
                 silent_threshold,
                 track_tests,
                 json_tests,
-                scale_tests,
                 codec_tests,
                 motion_tests,
-                edit_positive_tests,
-                edit_negative_tests,
             ]
         )
 
